import { Component, EventEmitter, Input, Output } from '@angular/core';

export enum Status {
  Open = 'open',
  InProgress = 'in-progress',
  Completed = 'completed',
  Canceled = 'canceled'
}

export interface ProgressStatus {
  progressIndex: number,
  progressStatus: Status.InProgress | Status.Canceled,
  progressPercent: number,
}

@Component({
  selector: 'app-progress-stepper',
  imports: [],
  templateUrl: './progress-stepper.component.html',
  styleUrl: './progress-stepper.component.scss'
})
export class ProgressStepperComponent {
  selectedIndex = 0;
  STATUS = Status;

  @Input() steps: string[] = [];
  @Input() status: ProgressStatus = { progressIndex: 0, progressStatus: Status.Canceled, progressPercent: 50 }

  @Output() selectionChange: EventEmitter<number> = new EventEmitter();

  selectStep(index: number) {
    if (index <= this.status.progressIndex)
      this.selectedIndex = index;

    this.selectionChange.emit(index)
  }

  indexStatus(index: number): Status {
    if (index < this.status.progressIndex)
      return Status.Completed;
    else if (index === this.status.progressIndex)
      if (this.status.progressStatus === Status.InProgress)
        return Status.InProgress;
      else return Status.Canceled;
    else
      return Status.Open
  }

  statusToLabel(status: Status): string {
    switch (status) {
      case Status.Open: return 'Offen';
      case Status.InProgress: return 'In Bearbeitung';
      case Status.Canceled: return 'Verweigert';
      case Status.Completed: return 'Fertig';
    }
  }

  get getProgress(): string {
    return `${Math.min(Math.max(this.status.progressPercent, 0), 100)}%`
  }
}
