import { A, Z, ZERO, NINE, SPACE } from '@angular/cdk/keycodes';
import { Component, ElementRef, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { FormGroup, FormBuilder, ReactiveFormsModule } from '@angular/forms';
import { Subscription } from 'rxjs';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatRippleModule } from '@angular/material/core';

@Component({
  selector: 'app-select-filter',
  imports: [ReactiveFormsModule, MatProgressSpinnerModule, MatRippleModule],
  templateUrl: './select-filter.component.html',
  styleUrl: './select-filter.component.scss'
})
export class SelectFilterComponent {

  private searchFormValueChangesSubscription!: Subscription;
  @ViewChild('input', { static: true }) input!: ElementRef;

  @Input('array') array: any[] = [];
  @Input('showSpinner') showSpinner = true;
  @Input('noResultsMessage') noResultsMessage = 'No results';

  noResults = false;

  localSpinner = false;
  @Output() filteredReturn = new EventEmitter<any>();
  @Output() selectAll = new EventEmitter<void>();
  @Output() unselectAll = new EventEmitter<void>();

  public filteredItems: any = [];
  public searchForm: FormGroup;

  constructor(fb: FormBuilder) {
    this.searchForm = fb.group({
      value: ''
    });
  }

  ngOnInit() {
    this.searchFormValueChangesSubscription = this.searchForm.valueChanges.subscribe(change => {
      if (this.showSpinner) {
        this.localSpinner = true;
      }
      if (change.value) {
        // IF THE DISPLAY MEMBER INPUT IS SET WE CHECK THE SPECIFIC PROPERTY

        this.filteredItems = this.array.filter((name: string) => name.toLowerCase().includes(change.value.toLowerCase()));
        // OTHERWISE, WE CHECK THE ENTIRE STRING

        // NO RESULTS VALIDATION

        this.noResults = this.filteredItems == null || this.filteredItems.length === 0;


      } else {
        this.filteredItems = this.array.slice();
        this.noResults = false;
      }
      this.filteredReturn.emit(this.filteredItems);
      setTimeout(() => {
        if (this.showSpinner) {
          this.localSpinner = false;
        }
      }, 2000);
    });

    setTimeout(() => {
      this.input.nativeElement.focus();
    }, 500);
  }

  handleKeydown(event: KeyboardEvent) {
    // PREVENT PROPAGATION FOR ALL ALPHANUMERIC CHARACTERS IN ORDER TO AVOID SELECTION ISSUES
    if ((event.key && event.key.length === 1) ||
      (event.keyCode >= A && event.keyCode <= Z) ||
      (event.keyCode >= ZERO && event.keyCode <= NINE) ||
      (event.keyCode === SPACE)) {
      event.stopPropagation();
    }
  }
  ngOnDestroy() {
    this.filteredReturn.emit(this.array);
    this.searchFormValueChangesSubscription.unsubscribe();
  }
}
